import { authInstance, languageInstance, languageStore } from "@shared";
import { makeAutoObservable, runInAction } from "mobx";

export type Language = "ru" | "en" | "zh";

export type MultilingualContent = {
  [key in Language]: {
    name: string;
    description: string;
    address: string;
  };
};

export type Sight = {
  id: number;
  name: string;
  city_id: number;
  city: string;
  address: string;
  latitude: number;
  longitude: number;
  thumbnail: string | null;
  watermark_lu: string | null;
  watermark_rd: string | null;
  left_article: number;
  preview_media: string;
  video_preview: string | null;
};

export type CreateSight = {
  [key in Language]: {
    name: string;
    description: string;
    address: string;
  };
};

class SightsStore {
  sights: Sight[] = [];
  sight: Sight | null = null;
  createSight: CreateSight = {
    ru: { name: "", description: "", address: "" },
    en: { name: "", description: "", address: "" },
    zh: { name: "", description: "", address: "" },
  };

  constructor() {
    makeAutoObservable(this);
  }

  getSights = async () => {
    const response = await authInstance.get("/sight");
    runInAction(() => {
      this.sights = response.data;
    });
  };

  getSight = async (id: number) => {
    const response = await authInstance.get(`/sight/${id}`);

    runInAction(() => {
      this.sight = response.data;
    });
  };

  createSightAction = async (
    city: number,
    coordinates: { latitude: number; longitude: number }
  ) => {
    const id = (
      await authInstance.post("/sight", {
        name: this.createSight[languageStore.language].name,
        description: this.createSight[languageStore.language].description,
        address: this.createSight[languageStore.language].address,
        city_id: city,
        latitude: coordinates.latitude,
        longitude: coordinates.longitude,
      })
    ).data.id;

    const anotherLanguages = ["ru", "en", "zh"].filter(
      (language) => language !== languageStore.language
    );

    await languageInstance(anotherLanguages[0] as Language).patch(
      `/sight/${id}`,
      {
        name: this.createSight[anotherLanguages[0] as Language].name,
        description:
          this.createSight[anotherLanguages[0] as Language].description,
        address: this.createSight[anotherLanguages[0] as Language].address,
        city_id: city,
        latitude: coordinates.latitude,
        longitude: coordinates.longitude,
      }
    );

    await languageInstance(anotherLanguages[1] as Language).patch(
      `/sight/${id}`,
      {
        name: this.createSight[anotherLanguages[1] as Language].name,
        description:
          this.createSight[anotherLanguages[1] as Language].description,
        address: this.createSight[anotherLanguages[1] as Language].address,
        city_id: city,
        latitude: coordinates.latitude,
        longitude: coordinates.longitude,
      }
    );

    runInAction(() => {
      this.createSight = {
        ru: { name: "", description: "", address: "" },
        en: { name: "", description: "", address: "" },
        zh: { name: "", description: "", address: "" },
      };
    });
  };
  setCreateSight = (content: CreateSight) => {
    runInAction(() => {
      this.createSight = content;
    });
  };

  updateCreateSight = (
    language: Language,
    content: Partial<CreateSight[Language]>
  ) => {
    runInAction(() => {
      this.createSight[language] = {
        ...this.createSight[language],
        ...content,
      };
    });
  };

  clearCreateSight = () => {
    runInAction(() => {
      this.createSight = {
        ru: {
          name: "",
          description: "",
          address: "",
        },
        en: {
          name: "",
          description: "",
          address: "",
        },
        zh: {
          name: "",
          description: "",
          address: "",
        },
      };
    });
  };
}

export const sightsStore = new SightsStore();
